"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeOperation = void 0;
const execution_plans_1 = require("@takomo/execution-plans");
const stacks_commands_1 = require("@takomo/stacks-commands");
const util_1 = require("@takomo/util");
const create_deployment_target_variables_1 = require("./create-deployment-target-variables");
const executeOperationInternal = async (operation, input, account, credentialManager, io, ctx, configRepository, logger, expectNoChanges, prune) => {
    switch (operation) {
        case "deploy":
            return (0, stacks_commands_1.deployStacksCommand)({
                input: {
                    ...input,
                    expectNoChanges,
                },
                ctx,
                credentialManager,
                configRepository,
                io: io.createStackDeployIO(logger),
            });
        case "undeploy":
            return (0, stacks_commands_1.undeployStacksCommand)({
                input: {
                    ...input,
                    prune,
                },
                ctx,
                credentialManager,
                configRepository,
                io: io.createStackUndeployIO(logger),
            });
        default:
            throw new Error(`Unsupported operation: ${operation}`);
    }
};
const createExecutor = ({ outputFormat, configRepository, operation, ctx, io, expectNoChanges, prune, }) => {
    return async ({ timer, logger, state, commandPath, configSet, target, defaultCredentialManager, }) => {
        if (state.failed) {
            logger.debug("Cancel operation");
            timer.stop();
            return {
                outputFormat,
                timer,
                status: "CANCELLED",
                message: "Cancelled",
                success: false,
                results: [],
            };
        }
        try {
            const input = {
                outputFormat,
                timer,
                commandPath,
                ignoreDependencies: false,
                interactive: false,
            };
            const stacksConfigRepository = await configRepository.createStacksConfigRepository(configSet.name, configSet.legacy);
            const variables = (0, create_deployment_target_variables_1.createDeploymentTargetVariables)({ target, ctx });
            const credentialManager = target.data.executionRoleArn
                ? await defaultCredentialManager.createCredentialManagerForRole(target.data.executionRoleArn)
                : defaultCredentialManager;
            if (target.data.accountId) {
                const identity = await credentialManager.getCallerIdentity();
                if (identity.accountId !== target.data.accountId) {
                    throw new util_1.TakomoError(`Current credentials belong to AWS account ${identity.accountId}, but the deployment target can be deployed only to account: ${target.data.accountId}`);
                }
            }
            return await executeOperationInternal(operation, input, target.data, credentialManager, io, {
                ...ctx.commandContext,
                variables,
            }, stacksConfigRepository, logger, expectNoChanges, prune);
        }
        catch (error) {
            logger.error("An error occurred", error);
            timer.stop();
            return {
                error,
                outputFormat,
                timer,
                status: "FAILED",
                message: "Failed",
                success: false,
                results: [],
            };
        }
    };
};
const executeOperation = async (holder) => {
    const { timer, io, plan, input: { concurrentTargets, outputFormat, operation, expectNoChanges, prune, }, ctx, } = holder;
    io.info("Execute operation");
    const executor = createExecutor({
        outputFormat,
        configRepository: ctx.configRepository,
        ctx,
        operation,
        io,
        expectNoChanges,
        prune,
    });
    return (0, execution_plans_1.executeConfigSetPlan)({
        ctx,
        plan,
        executor,
        concurrentTargets,
        logger: io,
        state: { failed: false },
        timer: timer.startChild("execute"),
        defaultCredentialManager: ctx.credentialManager,
        targetListenerProvider: io.createTargetListener,
    });
};
exports.executeOperation = executeOperation;
//# sourceMappingURL=execute.js.map